==> 0.3.0.tar.gz: lib/src/iroute_state.dart <==
import 'package:flutter/material.dart';
import 'url_parser.dart';
import 'iroute_common.dart';

///单一的路由状态
///push,pop,setNewPath三处要通知
class IRouteState extends ChangeNotifier {
  //singleton模式，用于处理历史记录者，全局只需要一个
  static final IRouteState _instance = IRouteState._internal();
  static bool inited = false;
  late Map<String, RouteBuilder> _routes;
  get routes => _routes;

  RouteBuilder? home;

  ///初始化，用于指定routes，routes是必须有得
  static init(Map<String, RouteBuilder> routes) {
    assert(!inited, 'init could run only once');
    inited = true;
    _instance._routes = routes;
    if (_instance.routes.containsKey('_'))
      _instance.home = _instance.routes['_'];
  }

  factory IRouteState() => _instance;
  IRouteState._internal();

  //页面的列表
  //无需每次维护，只有setNewRoutePath、push、pop会改变它
  //不可为空
  List<Page> _stack = [];
  List<Page> get stack => _stack;

  bool get canPop => _stack.length > 1;

  String? get currentConfiguration =>
      _stack.isNotEmpty ? _stack.last.name : null;

  void push(String path) {
    //if (path == _stack.last.name) return;
    _stack.add(_getPage(path));
    notifyListeners();
  }

  void pop() {
    if (!canPop) return;
    _stack.removeLast();
    notifyListeners();
  }

  Future<void> setNewRoutePath(String path) async {
    //if (currentConfiguration != null && path == currentConfiguration) return;
    push(path);
  }

  //根据url创建一个页面
  Page _getPage(path) {
    String pattern = UrlParser.getPattern(path);
    var params = UrlParser.getParam(path);
    if (!routes.containsKey(pattern)) {
      throw '$pattern not exist';
    }

    Widget widget =
        Builder(builder: (context) => routes[pattern]!(context, params));
    if (home != null)
      widget = Builder(
          builder: (context) => home!(context, {
                'child': ((context) =>
                    this.routes[pattern]!(context, params))(context)
              }));
    return MaterialPage(
        // key: ValueKey(path),
        name: path,
        child: widget);
  }
}

==> 0.4.0-dev.2.tar.gz: lib/src/iroute_state.dart <==
import 'package:flutter/material.dart';
import 'url_parser.dart';
import 'iroute_common.dart';

///单一的路由状态
///push,pop,setNewPath三处要通知
class IRouteState extends ChangeNotifier {
  //singleton模式，用于处理历史记录者，全局只需要一个
  static final IRouteState _instance = IRouteState._internal();
  static bool inited = false;
  late Map<String, RouteBuilder> _routes;
  get routes => _routes;

  RouteBuilder? home;

  ///初始化，用于指定routes，routes是必须有得
  static init(Map<String, RouteBuilder> routes) {
    assert(!inited, 'init could run only once');
    inited = true;
    _instance._routes = routes;
    if (_instance.routes.containsKey('_'))
      _instance.home = _instance.routes['_'];
  }

  factory IRouteState() => _instance;
  IRouteState._internal();

  //页面的列表
  //无需每次维护，只有setNewRoutePath、push、pop会改变它
  //不可为空
  List<Page> _stack = [];
  List<Page> get stack => _stack;

  bool get canPop => _stack.length > 1;

  String? get currentConfiguration {
    if (_stack.isEmpty) _stack.add(_getPage('/'));
    return _stack.last.name;
  }

  void push(String path) {
    //if (path == _stack.last.name) return;
    _stack.add(_getPage(path));
    notifyListeners();
  }

  void pop() {
    if (!canPop) return;
    _stack.removeLast();
    notifyListeners();
  }

  Future<void> setNewRoutePath(String path) async {
    //if (currentConfiguration != null && path == currentConfiguration) return;
    push(path);
  }

  //根据url创建一个页面
  Page _getPage(path) {
    String pattern = UrlParser.getPattern(path);
    var params = UrlParser.getParam(path);
    if (!routes.containsKey(pattern)) {
      throw '$pattern not exist';
    }

    Widget widget =
        Builder(builder: (context) => routes[pattern]!(context, params));
    if (home != null)
      widget = Builder(
          builder: (context) => home!(context, {
                'child': ((context) =>
                    this.routes[pattern]!(context, params))(context)
              }));
    return MaterialPage(key: UniqueKey(), name: path, child: widget);
  }
}
